#include <R.h>
#include <Rinternals.h>
#include "nag_rwrapper_utils.h"

/* Wrappers for the user supplied functions for the constraints and objective functions */
#ifdef __cplusplus
extern "C" {
#endif
  void NAG_CALL we04uccL_con(Integer n, Integer ncnlin, Integer needc[], double x[], double conf[], double conjac[], Nag_Comm *comm);
  void NAG_CALL we04uccL_obj(Integer n, double x[], double *objf, double objgrd[], Nag_Comm *comm);
  void NAG_CALL we04uccF_con(Integer n, Integer ncnlin, Integer needc[], double x[], double conf[], double conjac[], Nag_Comm *comm);
  void NAG_CALL we04uccF_obj(Integer n, double x[], double *objf, double objgrd[], Nag_Comm *comm);
#ifdef __cplusplus
}
#endif

/* Utility functions */
void we04ucc_clean_up(void **pv1, double *pd1, double *pd2, double *pd3, double *pd4, double *pd5, Nag_E04_Opt *options);
int we04ucc_handle_options(SEXP optlist, Nag_E04_Opt *(*options));

/* Length of array to hold error messages, no checks are made to ensure this is large enough */
#define LEN_ERR_MSG 100
/* Flag indicating column major order (!=0) or row major order (0) */
#define IS_COLUMN_MAJOR_ORDER 0

SEXP we04ucc(SEXP psize, SEXP a, SEXP lower, SEXP upper, SEXP objfun, SEXP confun, SEXP initial, SEXP optlist, SEXP rho) {
  /* we04ucc: Wrapper for NAG C library routine e04ucc.

     On entry:
     psize   - Vector containing the problem size, psize = c(N,NCLIN,NCNLIN),
               where N = number of variables, NCLIN = number linear constraints
               NCNLIN = number of non-linear constraints
     a       - (NCLIN x N) Matrix of linear constraints. If NCLIN = 0, a is not referenced, 
     lower   - List of lower constraint limits. List should contain three elements, the first
               a vector of N limits for simple constraints, the second a vector of NCLIN
               limits for the linear constraints and the third a vector of NCNLIN limits for
               the non-linear constraints
     upper   - List of upper constraint limits. This should be in the same form as lower
     objfun  - Either an R function or the body of an R function that calculates the objective 
               function and its gradients.
     confun  - Either an R function or the body of an R function that calculates the non-linear 
               constraints and corresponding derivatives.
     initial - Vector of N initial values
     optlist - List of options, in format that can be read by we04ucc_handle_options
     rho     - R environment in which the two functions objfun and confun are to be evaluated.


     Returns:
     A list with entries: 'objf', 'x' and 'g', which correspond to the e04ucc arguments of the
     same name. If, on entry, optlist is a list (irrespective of whether it has any valid options), 
     then 'h', 'iter', 'nf', 'conf' and 'conjac' are also returned. 'conf' and 'conjac' are only
     returned if NCNLIN > 0. These last 5 parameters correspond to the optional arguments of the same
     name held in the e04ucc argument options.

     Notes:
     Much of the parameter checking done in this routine could have been done in the R wrapper
     that calls we04ucc (and would probably have been simpler to do). 
  */
  Integer nclin, n, ncnlin, nbounds, k, lans, used_opt, probsize[3], cmajor, failed;
  Integer tda = 0;
  SEXP ans, names, lower_s, lower_l, lower_n, upper_s, upper_l, upper_n; 
  double objf;
  double *ca = 0, *bu = 0, *bl = 0, *x = 0, *g = 0;
  NagError fail;
  Nag_Comm comm;
  void **p = 0;
  Nag_E04_Opt options, *poptions = 0;
  char err_msg[LEN_ERR_MSG];

  /* Initialise the NAG error structure */
  INIT_FAIL(fail);

  /* Set type of matrix we are working with (when calling NAG C library this will be row major) */
  cmajor = IS_COLUMN_MAJOR_ORDER;

  /* Initial parameter checks */
  if ((failed = nag_extractIntegerVector(3,psize,&probsize[0]))) {
    sprintf(err_msg,"'psize' must be a numeric vector of length 3 (code = %ld)", (long) failed);
    error(err_msg);
  }
  /* NB: Need to use isNewList in preference to isList to test for lists */
  if (!isNewList(lower)) {
    error("'lower' must be a list with three elements");
  }
  if (length(lower) != 3) {
    error("'lower' must be a list with three elements");
  }
  if (!isNewList(upper)) {
    error("'upper' must be a list with three elements");
  }
  if (length(upper) != 3) {
    error("'upper' must be a list with three elements");
  }
  if (!isLanguage(confun) && !isFunction(confun)) {
    error("'confun' must be a function or the body of a function");
  }
  if (!isLanguage(objfun) && !isFunction(objfun)) {
    error("'objfun' must be a function or the body of a function");
  }
  if (isLanguage(objfun) != isLanguage(confun)) {
    error("'objfun' and 'confun' should either both be functions, or both the body of a function");
  }
  if (!isEnvironment(rho)) {
    error("'rho' must be an environment");
  }
  /* End of initial parameter checks */

  /* Get the problem size */
  n = probsize[0];
  nclin = probsize[1];
  ncnlin = probsize[2];
  if (n < 0 || nclin < 0 || ncnlin < 0) {
    error("'psize' must be a numeric vector of positive values");
  }

  /* Check the size of the linear constraint matrix */
  if (nclin > 0) {
    /* Allocate memory for the matrix of linear constraints */
    if (!(ca = (double *) malloc(nclin*n*sizeof(double)))) {
      we04ucc_clean_up(p,ca,bu,bl,x,g,poptions);
      error("memory allocation error");
    }

    /* Get the user supplied matrix A, into CA */
    if ((failed = nag_extractDoubleMatrix(cmajor,nclin,n,a,ca))) {
      we04ucc_clean_up(p,ca,bu,bl,x,g,poptions);
      sprintf(err_msg,"'a' must be a %ld x %ld numeric matrix (code = %ld)", (long) nclin, (long) n, (long) failed);
      error(err_msg);
    }

    tda = n;
  }

  /* Allocate memory for the bounds */
  nbounds = n + nclin + ncnlin;
  if (!(bu = (double *) malloc(nbounds*sizeof(double))) ||
      !(bl = (double *) malloc(nbounds*sizeof(double)))) {
    we04ucc_clean_up(p,ca,bu,bl,x,g,poptions);
    error("memory allocation error");
  }

  /* Copy and check the simple bounds */
  PROTECT(lower_s = VECTOR_ELT(lower,0));
  if ((failed = nag_extractDoubleVector(n,lower_s,bl))) {
    we04ucc_clean_up(p,ca,bu,bl,x,g,poptions);
    sprintf(err_msg,"First element of 'lower' must be a numeric vector of length %ld (code = %ld)", (long) n, (long) failed);
    error(err_msg);
  }

  PROTECT(upper_s = VECTOR_ELT(upper,0));
  if ((failed = nag_extractDoubleVector(n,upper_s,bu))) {
    we04ucc_clean_up(p,ca,bu,bl,x,g,poptions);
    sprintf(err_msg,"First element of 'upper' must be a numeric vector of length %ld (code = %ld)", (long) n, (long) failed);
    error(err_msg);
  }
  k = n;
  UNPROTECT(2);
  
  /* Copy and check the linear bounds (if required) */
  if (nclin > 0) {
    PROTECT(lower_l = VECTOR_ELT(lower,1));
    if ((failed = nag_extractDoubleVector(nclin,lower_l,&bl[k]))) {
      we04ucc_clean_up(p,ca,bu,bl,x,g,poptions);
      sprintf(err_msg,"Second element of 'lower' must be a numeric vector of length %ld (code = %ld)", (long) nclin, (long) failed);
      error(err_msg);
    }

    PROTECT(upper_l = VECTOR_ELT(upper,1));
    if ((failed = nag_extractDoubleVector(nclin,upper_l,&bu[k]))) {
      we04ucc_clean_up(p,ca,bu,bl,x,g,poptions);
      sprintf(err_msg,"Second element of 'upper' must be a numeric vector of length %ld (code = %ld)", (long) nclin, (long) failed);
      error(err_msg);
    }
    k += nclin;
    UNPROTECT(2);
  }

  /* Copy and check the non-linear bounds (if required) */
  if (ncnlin > 0) {
    PROTECT(lower_n = VECTOR_ELT(lower,2));
    if ((failed = nag_extractDoubleVector(ncnlin,lower_n,&bl[k]))) {
      we04ucc_clean_up(p,ca,bu,bl,x,g,poptions);
      sprintf(err_msg,"Third element of 'lower' must be a numeric vector of length %ld (code = %ld)", (long) ncnlin, (long) failed);
      error(err_msg);
    }

    PROTECT(upper_n = VECTOR_ELT(upper,2));
    if ((failed = nag_extractDoubleVector(ncnlin,upper_n,&bu[k]))) {
      we04ucc_clean_up(p,ca,bu,bl,x,g,poptions);
      sprintf(err_msg,"Third element of 'upper' must be a numeric vector of length %ld (code = %ld)", (long) ncnlin, (long) failed);
      error(err_msg);
    }
    UNPROTECT(2);
  }

  /* Allocate memory for the communication array, to hold confun, objfun and rho */
  if (!(p = (void **) malloc(3*sizeof(void *)))) {
    we04ucc_clean_up(p,ca,bu,bl,x,g,poptions);
    error("Memory allocation error");
  }

  /* Convert function pointers confun and objfun and rho into Nag_Comm structure */
  p[0] = (void *) confun;
  p[1] = (void *) objfun;
  p[2] = (void *) rho;
  comm.p = p;

  /* Handle options */
  poptions = &options;
  used_opt = we04ucc_handle_options(optlist, &poptions);

  /* Allocate rest of memory */
  if (!(x = (double *) malloc(n*sizeof(double))) ||
      !(g = (double *) malloc(n*sizeof(double)))) {
    we04ucc_clean_up(p,ca,bu,bl,x,g,poptions);
    error("Memory allocation error");
  }    

  /* Get the initial values */
  if ((failed = nag_extractDoubleVector(n,initial,x))) {
    we04ucc_clean_up(p,ca,bu,bl,x,g,poptions);
    sprintf(err_msg,"'initial' must be a numeric vector of length %ld (code = %ld)", (long) n, (long) failed);
    error(err_msg);
  }

  /* Call the NAG routine */
  if (isLanguage(objfun)) {
    /* Run version that expects a body of a function */
    e04ucc(n,nclin,ncnlin,ca,tda,bl,bu,we04uccL_obj,we04uccL_con,x,&objf,g,poptions,&comm,&fail);

  } else { 
    /* Run version that expects the function itself */
    e04ucc(n,nclin,ncnlin,ca,tda,bl,bu,we04uccF_obj,we04uccF_con,x,&objf,g,poptions,&comm,&fail);
  }

  /* Error returns from NAG routine */
  if (fail.code != NE_NOERROR) {
    we04ucc_clean_up(p,ca,bu,bl,x,g,poptions);
    error(fail.message);
  }

  /* Set the number of values we are returning */
  lans = 3;
  if (used_opt) {
    lans = lans + 3;
    if (ncnlin > 0) {
      lans = lans + 2;
    }
  }

  /* Allocate memory for output and the names of the output */
  PROTECT(ans = allocVector(VECSXP,lans));
  PROTECT(names = allocVector(STRSXP, lans));

  /* Get the list of results that are always returned */
  SET_VECTOR_ELT(ans,0,nag_d2r(1,&objf));
  SET_STRING_ELT(names,0,mkChar("objf"));

  SET_VECTOR_ELT(ans,1,nag_d2r(n,x));
  SET_STRING_ELT(names,1,mkChar("x"));

  SET_VECTOR_ELT(ans,2,nag_d2r(n,g));
  SET_STRING_ELT(names,2,mkChar("g"));

  /* These values are only returned if a non-null value is given for the optional parameters
     list. Even if the supplied list has no valid options in it
  */
  if (used_opt) {
    SET_VECTOR_ELT(ans,3,nag_md2r(cmajor,n,n,options.h));
    SET_STRING_ELT(names,3,mkChar("h"));

    SET_VECTOR_ELT(ans,4,nag_i2r(1,&options.iter));
    SET_STRING_ELT(names,4,mkChar("iter"));

    SET_VECTOR_ELT(ans,5,nag_i2r(1,&options.nf));
    SET_STRING_ELT(names,5,mkChar("nf"));

    if (ncnlin > 0) {
      SET_VECTOR_ELT(ans,6,nag_d2r(ncnlin,options.conf));
      SET_STRING_ELT(names,6,mkChar("conf"));

      SET_VECTOR_ELT(ans,7,nag_md2r(cmajor,ncnlin,n,options.conjac));
      SET_STRING_ELT(names,7,mkChar("conjac"));
    }
  }

  /* Set the names attribute */
  setAttrib(ans,R_NamesSymbol,names);

  /* Free up allocated memory */
  we04ucc_clean_up(p,ca,bu,bl,x,g,poptions);

  /* Unprotect the ans and names objects */
  UNPROTECT(2);

  return(ans);
}

/*********************************************************************************************/
/* Wrappers for callback functions, when user supplies the body of a function */
void NAG_CALL we04uccL_obj(Integer n, double x[], double *objf, double objgrd[], Nag_Comm *comm) {
  /* we04uccL_obj: Wrapper for the objective function callback function for E04UCC. For use
     when the user has supplied the body of a function (as opposed to the function itself).
     Used when calling a NAG C routine

     On entry:
     n      - The number of variables
     x      - The value at which the objective function and gradients should be evaluated, x[n]
     comm   - Communication array, three elements of which are used
              comm->flag  - If 0 or 2, then objective function required, if 2 the gradient is also required
              comm->first - Set to Nag_TRUE if this is the first time that this function has been called
              comm->p[1]  - points to the function (or code segment) that the user supplied for 
                            calculating the objective function
              comm->p[2]  - points to the environment the function is evaluated in

     On exit:
     objf   - The objective function evaluated at x (if comm->flag = 0 or 2)
     objgrd - The gradients evaluated at x (if comm->flag = 2), objgrd[n]
     comm   - Communication array
              comm->flag - Set to -1000 if user function returned a list with an element called
                           "terminate"
                           Set to a different -ve value if the user function returned an invalid
                           list, otherwise unchanged
  */
  SEXP objfun, rho, ans, tobj, tgrad, tstop;
  Integer failed;
  void **p;
  p = (void **) comm->p;

  /* Get the pointer to the R objective function */
  objfun = (SEXP) p[1];
  /* Get the pointer to the R environment */
  rho = (SEXP) p[2];

  /* Makes n, x, objgrd, flag and first available to the function as a variables of the same name
     NB: Need to make current values of conjac available, in case the user can only supply a few values for the
     derivatives
  */
  defineVar(install("n"), nag_i2r(1,&n), rho);
  defineVar(install("x"), nag_d2r(n,x), rho);
  defineVar(install("objgrd"), nag_d2r(n,objgrd), rho);
  defineVar(install("flag"), nag_i2r(1,&comm->flag), rho);
  defineVar(install("first"), nag_l2l(1,&comm->first), rho);

  /* Get the results from evaluating the function in R */
  PROTECT(ans = eval(objfun,rho));

  /* Check to see if user has included an element in the list called "terminate" */
  PROTECT(tstop = nag_getListElement(ans, "terminate"));
  if (tstop != R_NilValue) {
    /* User wants us to stop */
    comm->flag = -1000;

  } else {
    /* Expecting the result as a list, entries "objf" and "objgrd", holding the
       objective function and the gradients respectively.
       
       Could just use the first element in list as objf and second as objgrd, but
       this demonstrates use of nag_getListElement routine.
    */ 
    if (comm->flag == 0 || comm->flag == 2) {
      PROTECT(tobj = nag_getListElement(ans, "objf"));
      if (tobj != R_NilValue) {
        *objf = REAL(tobj)[0];
        if (!R_FINITE(*objf)) {
          /* Returned value is not valid, so kill the optimiser */
          comm->flag = -1;
        }
      }
      UNPROTECT(1);
    }

    if (comm->flag == 2) {
      PROTECT(tgrad = nag_getListElement(ans, "objgrd"));
      if (tgrad != R_NilValue) {
        if ((failed = nag_extractDoubleVector(n,tgrad,objgrd))) {
          /* Returned value is not valid, so kill the optimiser */
          comm->flag = -(failed + 10);
        }
      }
      UNPROTECT(1);
    }
  }

  UNPROTECT(2);
}

void NAG_CALL we04uccL_con(Integer n, Integer ncnlin, Integer needc[], double x[], double conf[], double conjac[], Nag_Comm *comm) {
  /* we04uccL_obj: Wrapper for the constraint function callback function for E04UCC. For use
     when the user has supplied the body of a function (as opposed to the function itself).
     Used when calling a NAG C routine

     On entry:
     n      - The number of variables
     ncnlin - The number of non-linear constraints
     needc  - If needc[i] > 0, then values for (i+1)'th constraint is required, needc[ncnlin]
     x      - The value at which the constraints and gradients should be evaluated, x[n]
     comm   - Communication array, three elements of which are used
              comm->flag  - If 0 or 2, then constraints required, if 2 the gradient is also required
              comm->first - Set to Nag_TRUE if this is the first time that this function has been called
              comm->p[0]  - points to the function (or code segment) that the user supplied for 
                            calculating the constraint function
              comm->p[2]  - points to the environment the function is evaluated in

     On exit:
     conf   - The constraints evaluated at x (if comm->flag = 0 or 2), conf[ncnlin]
     conjac - The gradients evaluated at x (if comm->flag = 2), conjac[ncnlin*n]
     comm   - Communication array
              comm->flag - Set to -1000 if user function returned a list with an element called
                           "terminate"
                           Set to a different -ve value if the user function returned an invalid
                           list, otherwise unchanged
  */
  SEXP confun, rho, ans, tconf, tconjac, tstop;
  Integer failed, cmajor;
  void **p;
  p = (void **) comm->p;

  /* Get the pointer to the R constraint function */
  confun = (SEXP) p[0];
  /* Get the pointer to the R environment */
  rho = (SEXP) p[2];

  /* Makes n, ncnlin, needc, x, conjac, flag and first available to the function as a variables of the same name.
     NB: Need to make current values of conjac available, in case the user can only supply a few values for the
     derivatives
  */
  cmajor = IS_COLUMN_MAJOR_ORDER;
  defineVar(install("n"), nag_i2r(1,&n), rho);
  defineVar(install("ncnlin"), nag_i2r(1,&ncnlin), rho);
  defineVar(install("needc"), nag_i2r(ncnlin,needc), rho);
  defineVar(install("x"), nag_d2r(n,x), rho);
  defineVar(install("conjac"), nag_md2r(cmajor,ncnlin,n,conjac), rho);
  defineVar(install("flag"), nag_i2r(1,&comm->flag), rho);
  defineVar(install("first"), nag_l2l(1,&comm->first), rho);

  /* Get the results from evaluating the function in R */
  PROTECT(ans = eval(confun,rho));

  /* Check to see if user has included an element in the list called "terminate" */
  PROTECT(tstop = nag_getListElement(ans, "terminate"));
  if (tstop != R_NilValue) {
    /* User wants us to stop */
    comm->flag = -1000;

  } else {
    /* Expecting the result as a list, entries "conf" and "conjac", holding the
       constraints and their derivatives respectively
    */
    if (comm->flag == 0 || comm->flag == 2) {
      PROTECT(tconf = nag_getListElement(ans, "conf"));
      if (tconf != R_NilValue) {
        if ((failed = nag_extractDoubleVector(ncnlin,tconf,conf))) {
          /* Returned value is not valid, so kill the optimiser */
          comm->flag = -(failed + 20);
        }
      }
      UNPROTECT(1);
    }
    
    if (comm->flag == 2) {
      PROTECT(tconjac = nag_getListElement(ans, "conjac"));
      if (tconjac != R_NilValue) {
        cmajor = IS_COLUMN_MAJOR_ORDER;
        if ((failed = nag_extractDoubleMatrix(cmajor,ncnlin,n,tconjac,conjac))) {
          /* Returned value is not valid, so kill the optimiser */
          comm->flag = -(failed + 30);
        }
      }
      UNPROTECT(1);
    }
  }
  
  UNPROTECT(2);
}
/* End of wrappers for callback functions, when user supplies the body of a function */
/*********************************************************************************************/


/*********************************************************************************************/
/* Wrappers for callback functions, when user supplies the function itself */
void NAG_CALL we04uccF_obj(Integer n, double x[], double *objf, double objgrd[], Nag_Comm *comm) {
  /* we04uccF_obj: Wrapper for the objective function callback function for E04UCC. For use
     when the user has supplied the function itself (as opposed to the body of a function).
     Used when calling a NAG C routine

     On entry:
     n      - The number of variables
     x      - The value at which the objective function and gradients should be evaluated, x[n]
     comm   - Communication array, three elements of which are used
              comm->flag  - If 0 or 2, then objective function required, if 2 the gradient is also required
              comm->first - Set to Nag_TRUE if this is the first time that this function has been called
              comm->p[1]  - points to the function (or code segment) that the user supplied for 
                            calculating the objective function
              comm->p[2]  - points to the environment the function is evaluated in

     On exit:
     objf   - The objective function evaluated at x (if comm->flag = 0 or 2)
     objgrd - The gradients evaluated at x (if comm->flag = 2), objgrd[n]
     comm   - Communication array
              comm->flag - Set to -1000 if user function returned a list with an element called
                           "terminate"
                           Set to a different -ve value if the user function returned an invalid
                           list, otherwise unchanged
  */
  SEXP objfun, rho, ans, tobj, tgrad, tstop, R_fcall;
  Integer failed;
  void **p;
  p = (void **) comm->p;

  /* Get the pointer to the R objective function */
  objfun = (SEXP) p[1];
  /* Get the pointer to the R environment */
  rho = (SEXP) p[2];

  /* In this case objfun points to an actual function that takes 5 arguments */
  PROTECT(R_fcall = lang6(objfun,
                          nag_i2r(1,&n),
                          nag_d2r(n,x), 
                          nag_d2r(n,objgrd), 
                          nag_i2r(1,&comm->flag), 
                          nag_l2l(1,&comm->first)));

  /* Get the results from evaluating the function in R */
  PROTECT(ans = eval(R_fcall,rho));

  /* No longer need R_fcall */
  UNPROTECT(1);

  /* Check to see if user has included an element in the list called "terminate" */
  PROTECT(tstop = nag_getListElement(ans, "terminate"));
  if (tstop != R_NilValue) {
    /* User wants us to stop */
    comm->flag = -1000;

  } else {
    /* Expecting the result as a list, entries "objf" and "objgrd", holding the
       objective function and the gradients respectively.
       
       Could just use the first element in list as objf and second as objgrd, but
       this demonstrates use of nag_getListElement routine.
    */ 
    if (comm->flag == 0 || comm->flag == 2) {
      PROTECT(tobj = nag_getListElement(ans, "objf"));
      if (tobj != R_NilValue) {
        *objf = REAL(tobj)[0];
        if (!R_FINITE(*objf)) {
          /* Returned value is not valid, so kill the optimiser */
          comm->flag = -1;
        }
      }
      UNPROTECT(1);
    }

    if (comm->flag == 2) {
      PROTECT(tgrad = nag_getListElement(ans, "objgrd"));
      if (tgrad != R_NilValue) {
        if ((failed = nag_extractDoubleVector(n,tgrad,objgrd))) {
          /* Returned value is not valid, so kill the optimiser */
          comm->flag = -(failed + 10);
        }
      }
      UNPROTECT(1);
    }
  }

  UNPROTECT(2);
}

void NAG_CALL we04uccF_con(Integer n, Integer ncnlin, Integer needc[], double x[], double conf[], double conjac[], Nag_Comm *comm) {
  /* we04uccF_con: Wrapper for the constraint function callback function for E04UCC. For use
     when the user has supplied the function itself (as opposed to the body of a function).
     Used when calling a NAG C routine

     On entry:
     n      - The number of variables
     ncnlin - The number of non-linear constraints
     needc  - If needc[i] > 0, then values for (i+1)'th constraint is required, needc[ncnlin]
     x      - The value at which the constraints and gradients should be evaluated, x[n]
     comm   - Communication array, three elements of which are used
              comm->flag  - If 0 or 2, then constraints required, if 2 the gradient is also required
              comm->first - Set to Nag_TRUE if this is the first time that this function has been called
              comm->p[0]  - points to the function (or code segment) that the user supplied for 
                            calculating the constraint function
              comm->p[2]  - points to the environment the function is evaluated in

     On exit:
     conf   - The constraints evaluated at x (if comm->flag = 0 or 2), conf[ncnlin]
     conjac - The gradients evaluated at x (if comm->flag = 2), conjac[ncnlin*n]
     comm   - Communication array
              comm->flag - Set to -1000 if user function returned a list with an element called
                           "terminate"
                           Set to a different -ve value if the user function returned an invalid
                           list, otherwise unchanged
  */
  SEXP confun, rho, ans, tconf, tconjac, tstop, R_fcall;
  Integer failed, cmajor;
  void **p;
  p = (void **) comm->p;

  /* Get the pointer to the R constraint function */
  confun = (SEXP) p[0];
  /* Get the pointer to the R environment */
  rho = (SEXP) p[2];

  cmajor = IS_COLUMN_MAJOR_ORDER;

  /* In this case confun points to an actual function that takes 7 arguments */
  PROTECT(R_fcall = lang8(confun,
                          nag_i2r(1,&n),
                          nag_i2r(1,&ncnlin),
                          nag_i2r(ncnlin,needc),
                          nag_d2r(n,x), 
                          nag_md2r(cmajor,ncnlin,n,conjac),
                          nag_i2r(1,&comm->flag), 
                          nag_l2l(1,&comm->first)));

  /* Get the results from evaluating the function in R */
  PROTECT(ans = eval(R_fcall,rho));

  /* No longer need R_fcall */
  UNPROTECT(1);

  /* Check to see if user has included an element in the list called "terminate" */
  PROTECT(tstop = nag_getListElement(ans, "terminate"));
  if (tstop != R_NilValue) {
    /* User wants us to stop */
    comm->flag = -1000;

  } else {
    /* Expecting the result as a list, entries "conf" and "conjac", holding the
       constraints and their derivatives respectively
    */
    if (comm->flag == 0 || comm->flag == 2) {
      PROTECT(tconf = nag_getListElement(ans, "conf"));
      if (tconf != R_NilValue) {
        if ((failed = nag_extractDoubleVector(ncnlin,tconf,conf))) {
          /* Returned value is not valid, so kill the optimiser */
          comm->flag = -(failed + 20);
        }
      }
      UNPROTECT(1);
    }
    
    if (comm->flag == 2) {
      PROTECT(tconjac = nag_getListElement(ans, "conjac"));
      if (tconjac != R_NilValue) {
        cmajor = IS_COLUMN_MAJOR_ORDER;
        if ((failed = nag_extractDoubleMatrix(cmajor,ncnlin,n,tconjac,conjac))) {
          /* Returned value is not valid, so kill the optimiser */
          comm->flag = -(failed + 30);
        }
      }
      UNPROTECT(1);
    }
  }
  
  UNPROTECT(2);
}
/* End of wrappers for callback functions, when user supplies the function itself */
/*********************************************************************************************/

int we04ucc_handle_options(SEXP optlist, Nag_E04_Opt *(*options)) {
  /* we04ucc_handle_options: Take an R list, and extract from it any options for E04UCC.

    On entry:
    optlist  - An R list
    options  - Pointer to pointer for a variable of type Nag_E04_Opt

    On exit:
    options  - If optlist is not a list, then options is set to E04_DEFAULT and
               a value of 0 is returned.
               If optlist is a list, then the elements of options are set
               based on the names of the values, and the values themselves, in the list, 
               and a value of 1 is returned.

               So, optlist = list(list=TRUE,f_prec=0.3), would set the option "list" to 
               Nag_TRUE, and the option "f_prec" to 0.3. Only the first entry of each 
               entry in optlist is used. The names are the same as listed in the e04ucc doc
               (all lower case). For print levels, 0 = Nag_NoPrint, 1 = Nag_Soln etc

    Returns:
    1 if options are used, 0 otherwise (i.e. optlist is not a list)
  */
  SEXP names, tentry;
  int i, ival;
  double dval;
  const char *this_name;

  if (!isNewList(optlist)) {
    *options = E04_DEFAULT;
    return 0;
  }

  /* Initialise the option structure */
  e04xxc((*options));

  /* Get the list of names from the list */
  PROTECT(names = getAttrib(optlist,R_NamesSymbol));
  for (i = 0; i < length(optlist); i++) {
    this_name = CHAR(STRING_ELT(names,i));
        
    /* Only interested in the first element, which is what asReal should
       convert to a double */
    PROTECT(tentry = VECTOR_ELT(optlist,i));
    if (!isString(tentry)) {
      dval = asReal(tentry);
      if (!R_FINITE(dval)) dval = 0.0;
      ival = (int) dval;
    } else {
      dval = -1.0;
      ival = -1;
    }
    
    if (!strcmp(this_name,"list")) {
      (*options)->list = (ival ? Nag_TRUE : Nag_FALSE);
      
    } else if (!strcmp(this_name,"print_level")) {
      if (ival == 0) {
        (*options)->print_level = Nag_NoPrint;
      } else if (ival == 1) {
        (*options)->print_level = Nag_Soln;
      } else if (ival == 2) {
        (*options)->print_level = Nag_Iter;
      } else if (ival == 3) {
        (*options)->print_level = Nag_Iter_Long;
      } else if (ival == 4) {
        (*options)->print_level = Nag_Soln_Iter;
      } else if (ival == 5) {
        (*options)->print_level = Nag_Soln_Iter_Long;
      } else if (ival == 6) {
        (*options)->print_level = Nag_Soln_Iter_Const;
      } else if (ival == 7) {
        (*options)->print_level = Nag_Soln_Iter_Full;
      }        
      
    } else if (!strcmp(this_name,"minor_print_level")) {
      if (ival == 0) {
        (*options)->minor_print_level = Nag_NoPrint;
      } else if (ival == 1) {
        (*options)->minor_print_level = Nag_Soln;
      } else if (ival == 2) {
        (*options)->minor_print_level = Nag_Iter;
      } else if (ival == 3) {
        (*options)->minor_print_level = Nag_Iter_Long;
      } else if (ival == 4) {
        (*options)->minor_print_level = Nag_Soln_Iter;
      } else if (ival == 5) {
        (*options)->minor_print_level = Nag_Soln_Iter_Long;
      } else if (ival == 6) {
        (*options)->minor_print_level = Nag_Soln_Iter_Const;
      } else if (ival == 7) {
        (*options)->minor_print_level = Nag_Soln_Iter_Full;
      }

    } else if (!strcmp(this_name,"outfile")) {
      
      strcpy((*options)->outfile,CHAR(STRING_ELT(tentry,0)));
      printf("OUTFILE:%s\n", (*options)->outfile);
     
    } else if (!strcmp(this_name,"obj_deriv")) {
      (*options)->obj_deriv = (ival ? Nag_TRUE : Nag_FALSE);

    } else if (!strcmp(this_name,"con_deriv")) {
      (*options)->con_deriv = (ival ? Nag_TRUE : Nag_FALSE);

    } else if (!strcmp(this_name,"verify_grad")) {
      if (ival == 0) {
        (*options)->verify_grad = Nag_NoCheck;
      } else if (ival == 1) {
        (*options)->verify_grad = Nag_SimpleCheck;
      } else if (ival == 2) {
        (*options)->verify_grad = Nag_CheckObj;
      } else if (ival == 3) {
        (*options)->verify_grad = Nag_CheckCon;
      } else if (ival == 4) {
        (*options)->verify_grad = Nag_CheckObjCon;
      } else if (ival == 5) {
        (*options)->verify_grad = Nag_XSimpleCheck;
      } else if (ival == 6) {
        (*options)->verify_grad = Nag_XCheckObj;
      } else if (ival == 7) {
        (*options)->verify_grad = Nag_XCheckCon;
      } else if (ival == 8) {
        (*options)->verify_grad = Nag_XCheckObjCon;
      }
    } else if (!strcmp(this_name,"print_deriv")) {
      if (ival == 0) {
        (*options)->print_deriv = Nag_D_NoPrint;
      } else if (ival == 1) {
        (*options)->print_deriv = Nag_D_Sum;
      } else if (ival == 2) {
        (*options)->print_deriv = Nag_D_Full;
      }

    } else if (!strcmp(this_name,"obj_check_start")) {
      (*options)->obj_check_start = ival;

    } else if (!strcmp(this_name,"obj_check_stop")) {
      (*options)->obj_check_stop = ival;

    } else if (!strcmp(this_name,"con_check_start")) {
      (*options)->con_check_start = ival;

    } else if (!strcmp(this_name,"con_check_stop")) {
      (*options)->con_check_stop = ival;

    } else if (!strcmp(this_name,"f_diff_int")) {
      (*options)->f_diff_int = dval;

    } else if (!strcmp(this_name,"c_diff_int")) {
      (*options)->c_diff_int = dval;

    } else if (!strcmp(this_name,"max_iter")) {
      (*options)->max_iter = ival;

    } else if (!strcmp(this_name,"minor_max_iter")) {
      (*options)->minor_max_iter = ival;

    } else if (!strcmp(this_name,"f_prec")) {
      (*options)->f_prec = dval;

    } else if (!strcmp(this_name,"optim_tol")) {
      (*options)->optim_tol = dval;

    } else if (!strcmp(this_name,"lin_feas_tol")) {
      (*options)->lin_feas_tol = dval;

    } else if (!strcmp(this_name,"nonlin_feas_tol")) {
      (*options)->nonlin_feas_tol = dval;

    } else if (!strcmp(this_name,"linesearch_tol")) {
      (*options)->linesearch_tol = dval;

    } else if (!strcmp(this_name,"step_limit")) {
      (*options)->step_limit = dval;

    } else if (!strcmp(this_name,"inf_bound")) {
      (*options)->inf_bound = dval;

    } else if (!strcmp(this_name,"inf_step")) {
      (*options)->inf_step = dval;

    } else if (!strcmp(this_name,"hessian")) {
      (*options)->hessian = (ival ? Nag_TRUE : Nag_FALSE);

    }
    /* All other options are ignored (this includes the valid ones
       start,conf,conjac,lambda and print_fun
    */

    UNPROTECT(1);
  }

  UNPROTECT(1);

  return 1;
}

void we04ucc_clean_up(void **pv1, double *pd1, double *pd2, double *pd3, double *pd4, double *pd5, Nag_E04_Opt *options) {
  /* we04ucc_clean_up: Frees any allocated memory.

     Routine frees the various pointers, if they are non-null, and calls e04xcz to free memory allocated
     to options (if it is not E04_DEFAULT, which is usually null)
  */
  NagError fail;

  if (pv1) free(pv1);
  if (pd1) free(pd1);
  if (pd2) free(pd2);
  if (pd3) free(pd3);
  if (pd4) free(pd4);
  if (pd5) free(pd5);
  if (options) {
    if (options != E04_DEFAULT) {
      INIT_FAIL(fail);
      e04xzc(options,"all",&fail);
    }
  }
  return;
}
